using UnityEngine;
using UnityEngine.Events;

namespace Mod
{
    public class ArmorSetBehaviour : MonoBehaviour
    {
        public string[] set;

        public void SpawnArmor()
        {
            for (int i = 0; i <= set.Length - 1; i++)
            {
                ArmorProperties armor = Mod.armor[set[i]];
                GameObject obj = ModAPI.CreatePhysicalObject(set[i], armor.sprite);
                obj.AddComponent<SerialiseInstructions>().OriginalSpawnableAsset = ModAPI.FindSpawnable("Rod");
                Destroy(obj.GetComponent<Optout>());

                obj.transform.position = transform.position;

                PhysicalBehaviour phys = GetComponent<PhysicalBehaviour>();
                PhysicalBehaviour objPhys = obj.GetComponent<PhysicalBehaviour>();

                objPhys.InitialMass = phys.InitialMass;
                objPhys.TrueInitialMass = phys.TrueInitialMass;
                objPhys.InitialGravityScale = phys.InitialGravityScale;
                objPhys.rigidbody.mass = phys.rigidbody.mass;
                objPhys.OverrideShotSounds = phys.OverrideShotSounds;

                ArmorBehaviour armorBehaviour = obj.AddComponent<ArmorBehaviour>();
                armorBehaviour.propName = set[i];
            }
        }
    }
    // all the armor stuff, makes armor attach and get damaged and stuff
    // stores the health of the armor
    public class ArmorBehaviour : MonoBehaviour
    {
        [SerializeField]
        public bool start = true;

        bool attached;
        [SerializeField]
        public LimbBehaviour attachedLimb;

        [SerializeField]
        public string propName;
        private ArmorProperties prop;
        private PhysicalBehaviour phys;

        private PhysicalProperties physicalProperties;

        [SerializeField]
        public float armorPoints;
        float initialPoints;
        [SerializeField]
        public float customInitialPoints;
        bool destroyed;
        [SerializeField]
        public bool damaged;
        [SerializeField]
        public bool durabilityDisabled;

        private float initialAbsorb;

        [SerializeField]
        public bool modified;
        [SerializeField]
        public float modifiedAbsorption;
        void Start()
        {
            phys = GetComponent<PhysicalBehaviour>();
            prop = Mod.armor[propName];

            SetProperties();
            ContextMenu();
            phys.RefreshOutline();

            if (attachedLimb)
                Attach(attachedLimb);

            phys.HoldingPositions = new Vector3[0];
            UpdateDamage();
        }
        void Update()
        {
            if (attached && !attachedLimb) // armor is deleted if whatever it is attached to is also deleted
                Destroy(gameObject);
        }
        void SetProperties()
        {
            SpriteRenderer sr = GetComponent<SpriteRenderer>();
            sr.sprite = prop.sprite;
            if (customInitialPoints == 0)
                initialPoints = prop.initialPoints;
            else
                initialPoints = customInitialPoints;
            physicalProperties = Instantiate(prop.physicalProperties);
            if (modified)
                initialAbsorb = modifiedAbsorption;
            else
                initialAbsorb = physicalProperties.BulletSpeedAbsorptionPower;
            if (start)
            {
                armorPoints = initialPoints;
                start = false;
            }
            phys.Properties = physicalProperties;
            if (prop.colliderPoints == null)
                gameObject.FixColliders();
            else
            {
                foreach (Collider2D collider in GetComponents<Collider2D>())
                    Destroy(collider);
                gameObject.AddComponent<PolygonCollider2D>().points = prop.colliderPoints;
            }
        }
        public void UpdateDamage()
        {
            armorPoints = Mathf.Max(armorPoints, 0);
            if (armorPoints == 0)
            {
                gameObject.layer = 10;
                destroyed = true;
            }
            else
            {
                gameObject.layer = 9;
                destroyed = false;
            }
            if (!destroyed)
                physicalProperties.BulletSpeedAbsorptionPower = initialAbsorb * (armorPoints / initialPoints);
            if (armorPoints == initialPoints)
                damaged = false;
        }
        void Shot(global::Shot shot)
        {
            if (durabilityDisabled)
                return;
            armorPoints -= shot.damage / 10;
            damaged = true;
            UpdateDamage();
        }
        void OnFragmentHit(float force)
        {
            if (attachedLimb)
            {
                force *= 5f;
                if (!destroyed)
                {
                    if (armorPoints - force < 0)
                        force -= armorPoints;
                    else
                        force /= 8f;
                }
                attachedLimb.SendMessage("OnFragmentHit", force, SendMessageOptions.DontRequireReceiver);
            }
            if (durabilityDisabled)
                return;
            armorPoints -= force;
            UpdateDamage();
        }
        void OnCollisionEnter2D(Collision2D collision)
        {
            LimbBehaviour limb = collision.gameObject.GetComponent<LimbBehaviour>();
            if (collision.gameObject.GetComponent<ArmorBehaviour>() || limb)
            {
                Nocollide(collision.gameObject);
                if (limb && !attached && limb.gameObject.name == prop.armorPiece)
                    Attach(limb);
            }
        }
        void Attach(LimbBehaviour limb)
        {
            GetComponent<Rigidbody2D>().angularVelocity = 0;
            SpriteRenderer sr = GetComponent<SpriteRenderer>();
            sr.sortingOrder = limb.gameObject.GetComponent<SpriteRenderer>().sortingOrder + 2;
            sr.sortingLayerName = limb.gameObject.GetComponent<SpriteRenderer>().sortingLayerName;
            attached = true;
            GetComponent<Rigidbody2D>().isKinematic = true;
            transform.parent = limb.transform;
            transform.localEulerAngles = new Vector3(0, 0, 0);
            transform.localPosition = prop.offset;
            transform.localScale = Vector3.one + prop.scaleOffset;
            transform.parent = null;

            BreatheBehaviour breathe = limb.gameObject.GetComponentInChildren<BreatheBehaviour>();
            SymptomBehaviour symptom = limb.gameObject.GetComponentInChildren<SymptomBehaviour>();
            if (prop.inhaleProtection && breathe)
                breathe.immune = true;
            if (prop.skinProtection && symptom)
                symptom.immune = true;

            FixedJoint2D joint = gameObject.AddComponent<FixedJoint2D>();
            joint.dampingRatio = 1;
            joint.frequency = 0;
            joint.connectedBody = limb.GetComponent<Rigidbody2D>();

            GetComponent<Rigidbody2D>().isKinematic = false;
            attachedLimb = limb;
        }
        void Nocollide(GameObject col)
        {
            NoCollide noCol = gameObject.AddComponent<NoCollide>();
            noCol.NoCollideSetA = GetComponents<Collider2D>();
            noCol.NoCollideSetB = col.GetComponents<Collider2D>();
        }
        void ContextMenu()
        {
            GetComponent<PhysicalBehaviour>().ContextMenuOptions.Buttons.Add(new ContextMenuButton("chekbut", "Check armor quality", "Check how damaged the armor is.", new UnityAction[1]
            {
                () =>
                {
                    if (armorPoints != 0)
                        ModAPI.Notify("Armor points " + Mathf.Round(armorPoints * 10) / 10 + " / " + initialPoints);
                    else
                        ModAPI.Notify("<color=red>Armor points 0 / " + initialPoints + "</color>");
                }
            }));
            GetComponent<PhysicalBehaviour>().ContextMenuOptions.Buttons.Add(new ContextMenuButton("repbut", "Repair armor", "Repair the armor.", new UnityAction[1]
            {
                () =>
                {
                    armorPoints = initialPoints;
                    UpdateDamage();
                    ModAPI.Notify("Armor repaired!");
                }
            }));
            GetComponent<PhysicalBehaviour>().ContextMenuOptions.Buttons.Add(new ContextMenuButton("durbut", "Modify durability", "Modify the max durability of the armor, or disable durability.", new UnityAction[1]
            {
                () =>
                {
                    DialogBox dialog = null;
                    dialog = DialogBoxManager.TextEntry("Input a value for max durability. This will repair the armor.", "", new DialogButton("Apply", true, new UnityAction[1]
                    {
                        () =>
                        {
                            if (dialog.EnteredText != "" && float.TryParse(dialog.EnteredText, out float result))
                            {
                                customInitialPoints = result;
                                initialPoints = result;
                                armorPoints = initialPoints;
                                UpdateDamage();
                            }
                            else if (dialog.EnteredText == "")
                                ModAPI.Notify("You didn't input anything.");
                            else
                                ModAPI.Notify("You didn't input a number.");
                        }
                    }),
                    new DialogButton(durabilityDisabled ? "Enable durability" : "Disable durability", true, new UnityAction[1]
                    {
                        () =>
                        {
                            if (durabilityDisabled)
                                durabilityDisabled = false;
                            else
                            {
                                armorPoints = initialPoints;
                                UpdateDamage();
                                durabilityDisabled = true;
                            }
                        }
                    }),
                    new DialogButton("Cancel", true, new UnityAction[1]
                    {
                        () => dialog.Close()
                    }));
                }
            }));
            GetComponent<PhysicalBehaviour>().ContextMenuOptions.Buttons.Add(new ContextMenuButton("absorbut", "Modify hardness", "Modify how well the armor stops bullets.", new UnityAction[1]
            {
                () =>
                {
                    DialogBox dialog = null;
                    dialog = DialogBoxManager.TextEntry("Input a value for absorption. Higher absorption values increase the amount of force from shots, so be careful. Current absorption is " + (modified ? modifiedAbsorption : physicalProperties.BulletSpeedAbsorptionPower) + ".", "", new DialogButton("Apply", true, new UnityAction[1]
                    {
                        () =>
                        {
                            if (dialog.EnteredText != "" && float.TryParse(dialog.EnteredText, out float result))
                            {
                                modifiedAbsorption = result;
                                initialAbsorb = result;
                                modified = true;
                                UpdateDamage();
                            }
                            else if (dialog.EnteredText == "")
                                ModAPI.Notify("You didn't input anything.");
                            else
                                ModAPI.Notify("You didn't input a number.");
                        }
                    }),
                    new DialogButton("Cancel", true, new UnityAction[1]
                    {
                        () => dialog.Close()
                    }));
                }
            }));
        }
    }
    // has the properties of armor like its health and sprite
    // used when creating armor
    public struct ArmorProperties
    {
        public ArmorProperties(string armorPiece, Sprite sprite, float armorPoints, PhysicalProperties physicalProperties, bool skin, bool inhale)
        {
            this.armorPiece = armorPiece;
            initialPoints = armorPoints;
            offset = Vector3.zero;
            scaleOffset = Vector3.zero;
            this.physicalProperties = physicalProperties;
            this.sprite = sprite;
            skinProtection = skin;
            inhaleProtection = inhale;
            colliderPoints = null;
        }
        public ArmorProperties(string armorPiece, Sprite sprite, Vector3 offset, Vector3 scaleOffset, float armorPoints, PhysicalProperties physicalProperties, bool skin, bool inhale)
        {
            this.armorPiece = armorPiece;
            initialPoints = armorPoints;
            this.offset = offset;
            this.scaleOffset = scaleOffset;
            this.physicalProperties = physicalProperties;
            this.sprite = sprite;
            skinProtection = skin;
            inhaleProtection = inhale;
            colliderPoints = null;
        }
        public ArmorProperties(string armorPiece, Sprite sprite, Vector3 offset, Vector3 scaleOffset, Vector2[] colliderPoints, float armorPoints, PhysicalProperties physicalProperties, bool skin, bool inhale)
        {
            this.armorPiece = armorPiece;
            initialPoints = armorPoints;
            this.offset = offset;
            this.scaleOffset = scaleOffset;
            this.physicalProperties = physicalProperties;
            this.sprite = sprite;
            skinProtection = skin;
            inhaleProtection = inhale;
            this.colliderPoints = colliderPoints;
        }

        public float initialPoints;
        public string armorPiece;

        public PhysicalProperties physicalProperties;

        public Vector2[] colliderPoints;
        public Vector3 offset;
        public Vector3 scaleOffset;

        public bool skinProtection;
        public bool inhaleProtection;

        public Sprite sprite;
    }
}