using System;
using System.Collections;
using UnityEngine;

namespace Mod
{
    public class BreatheBehaviour : SymptomBehaviour
    {
        public float lungEfficiency = 1f;
        public float breathTime = 1f;
        public float breathSize = .250f;
        public float breathRandomization = 0f;
        public bool regen;
        public float regenTime = 5f;
        public override void Start()
        {
            StartCoroutine(ContinuousBreathe());
            StartCoroutine(Regen());

            if (GetComponent<LimbBehaviour>())
                Destroy(this);
        }
        public override void FixedUpdate()
        {
            lungEfficiency = Mathf.Clamp01(lungEfficiency);
            if (lungEfficiency < 1)
            {
                // fastest suffocation can happen is 40 seconds, longest is 180 seconds
                float oxygenLossTime = lungEfficiency * 140f + 40;
                oxygenLossTime = Mathf.Clamp(oxygenLossTime, 90f, 360f);

                // cancels out oxygen regeneration then removes oxygen
                person.OxygenLevel -= (float) (Mathf.Pow(Mathf.Sin(Time.time * 3f), 16f) * 0.5 / 90.0 * (Time.deltaTime * 60.0));
                person.OxygenLevel -= Time.deltaTime / oxygenLossTime;
            }
            if (!immune)
                base.FixedUpdate();
        }
        public IEnumerator ContinuousBreathe()
        {
            while (true && !immune)
            {
                if (limb.Health <= 0)
                    yield return null;
                base.AbsorbGases(breathSize);
                DispenseAllGas(breathSize / 10);
                yield return new WaitForSeconds(breathTime + (UnityEngine.Random.value * breathRandomization));
            }
        }
        public override IEnumerator DoSymptom(Symptom symptom, Gas gas)
        {
            bool active = false;
            float delayPassed = 0;
            float timePassed = 0;
            float multiplier = 1;

            void UpdateMultiplier()
            {
                if (!gases.ContainsKey(gas))
                    multiplier = symptom.mild;
                else if (gases[gas] >= 4)
                    multiplier = symptom.severe;
                else if (gases[gas] >= 2)
                    multiplier = symptom.moderate;
                else
                    multiplier = symptom.mild;
                if (!exposed.Contains(gas))
                    multiplier *= symptom.nonexposedMultiplier;
            }
            activeSymptoms.Add(symptom);

            while (!active)
            {
                if (!gases.ContainsKey(gas))
                    break;
                if (UnityEngine.Random.value < symptom.chance * multiplier * Time.fixedDeltaTime * symptom.inhalationExposureMultiplier) // inhalation because breathebehaviour
                    active = true;
                UpdateMultiplier();
                yield return new WaitForFixedUpdate();
            }
            //float otherTime = 0; // debug
            while (active && !immune)
            {
                UpdateMultiplier();
                while (multiplier == 0)
                {
                    UpdateMultiplier();
                    if (!gases.ContainsKey(gas))
                        active = false;
                    yield return new WaitForFixedUpdate();
                }
                while (symptom.delay > 0 && multiplier != 0 && delayPassed < (symptom.delay / multiplier))
                {
                    UpdateMultiplier();
                    delayPassed += Time.fixedDeltaTime;
                    yield return new WaitForFixedUpdate();
                }
                //otherTime += Time.deltaTime;
                //if (otherTime >= 1) // debug
                //{
                //    Debug.Log(symptom.name);
                //    otherTime = 0;
                //}
                if (symptom.time >= 0 && !gases.ContainsKey(gas))
                {
                    timePassed += Time.fixedDeltaTime;
                    if (timePassed >= symptom.time)
                        active = false;
                }
                if (symptom.consciousnessEffect != null)
                    person.Consciousness = symptom.consciousnessEffect(person.Consciousness, multiplier);
                if (symptom.healthEffect != null)
                    limb.Health = symptom.healthEffect(limb, multiplier);
                if (symptom.lungEffect != null)
                    EffectLungs(symptom.lungEffect(lungEfficiency, multiplier));
                if (symptom.breathTimeEffect != null)
                    breathTime = symptom.breathTimeEffect(breathTime, multiplier);
                if (symptom.breathIntakeEffect != null)
                    breathSize = symptom.breathIntakeEffect(breathSize, multiplier);
                if (symptom.breathRandomizationEffect != null)
                    breathRandomization = symptom.breathRandomizationEffect(breathRandomization, multiplier);
                if (symptom.shockEffect != null)
                    person.ShockLevel = symptom.shockEffect(person.ShockLevel, multiplier);

                if (symptom.otherEffects != null)
                {
                    foreach (Action<GameObject, float> action in symptom.otherEffects)
                    {
                        action(limb.gameObject, multiplier);
                    }
                }
                yield return new WaitForFixedUpdate();
            }
            yield return new WaitWhile(() => gases.ContainsKey(gas)); // made while tired, make sure it actually fits this correctly
            activeSymptoms.Remove(symptom);
        }
        void EffectLungs(float effect)
        {
            if (effect <= lungEfficiency)
            {
                regenTime = 5f;
                regen = false;
            }
            lungEfficiency = Mathf.Clamp01(effect);
        }
        IEnumerator Regen()
        {
            while (true)
            {
                if (regen)
                {
                    if (lungEfficiency < 1)
                    {
                        lungEfficiency += 0.1f * Time.fixedDeltaTime;
                        lungEfficiency = Mathf.Max(0, 1);
                    }
                }
                else
                {
                    regenTime -= Time.fixedDeltaTime;
                    if (regenTime <= 0)
                    {
                        regenTime += 5;
                        regen = true;
                    }
                }
                yield return null;
            }
        }
    }
}