using System.Linq;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Events;

namespace Mod
{
    public class CanisterBehaviour : GasContainer
    {
        [SkipSerialisation]
        public Vector2 releasePoint;
        [SkipSerialisation]
        public Vector2 releaseVelocity;

        public float releaseRate = 0.5f;
        public float releaseAmount = 1;
        public float spawnRadius = 0.25f;
        public bool currentlyReleasing;
        bool bulletproof = false;
        
        [SkipSerialisation]
        PhysicalBehaviour physicalBehaviour;
        [SkipSerialisation]
        public AudioSource audio;

        public float windMin = 0.5f;
        public float windMax = 1f;

        public float fuse;
        bool fuseStarted;

        bool infiniteGas = false;
        string currentGas;

        int holes;
        Coroutine[] holeCoroutines = new Coroutine[2];

        int butPos;
        public bool CanReleaseGas()
        {
            if (GasAmount() != 0)
                return true;
            else
                return false;
        }

        public Gas NextGas()
        {
            return gases.Aggregate((x, y) => x.Value > y.Value ? x : y).Key;
        }

        public void Start()
        {
            ContextMenu();
            physicalBehaviour = GetComponent<PhysicalBehaviour>();
        }
        public void Use(ActivationPropagation _)
        {
            if (!currentlyReleasing && CanReleaseGas() && fuse <= 0)
            {
                currentlyReleasing = true;
                StartCoroutine(Releasing());
            }
            else if (fuse > 0)
            {
                if (fuseStarted)
                {
                    fuseStarted = false;
                }
                else
                {
                    fuseStarted = true;
                    StartCoroutine(Fuse());
                }
            }
            else
            {
                currentlyReleasing = false;
            }
        }
        public IEnumerator Fuse()
        {
            while (fuse > 0 && fuseStarted)
            {
                fuse -= Time.fixedDeltaTime;
                yield return new WaitForFixedUpdate();
            }
            if (fuse <= 0)
            {
                ModAPI.CreateParticleEffect("FuseBlown", transform.position);
                if (CanReleaseGas())
                {
                    currentlyReleasing = true;
                    StartCoroutine(Releasing());
                }
            }
        }
        public IEnumerator Releasing()
        {
            physicalBehaviour.PlayClipOnce(Mod.gasSounds[0], 0.25f);
            yield return new WaitForSeconds(releaseRate);
            audio.enabled = true;
            while (currentlyReleasing && CanReleaseGas())
            {
                ReleaseGas(NextGas(), releaseAmount);
                yield return new WaitForSeconds(releaseRate);
            }
            audio.enabled = false;
            physicalBehaviour.PlayClipOnce(Mod.gasSounds[2], 0.25f);
            currentlyReleasing = false;
        }
        public IEnumerator ReleaseFromHole(Shot shot)
        {
            float time = 0f;
            Vector2 point = transform.InverseTransformPoint(shot.point);
            Vector2 normal = transform.InverseTransformDirection(shot.normal);
            Rigidbody2D rb = GetComponent<Rigidbody2D>();
            while (CanReleaseGas())
            {
                while (time < 0.25f)
                {
                    time += Time.fixedDeltaTime;
                    yield return new WaitForFixedUpdate();
                }
                float random = Random.Range(1f, 1.5f);
                Vector2 velocity = new Vector2(random, random);
                ReleaseGas(NextGas(), Random.Range(5f, 7f), 0.1f, point, velocity * normal, 0, Random.Range(0.75f, 1.25f), 0.3f);
                time -= 0.25f;
            }
        }
        private void Shot(Shot shot)
        {
            if (bulletproof)
                return;
            Coroutine coroutine = StartCoroutine(ReleaseFromHole(shot));
            if (holes < holeCoroutines.Length)
            {
                holeCoroutines[holes] = coroutine;
                holes++;
            }
            else
            {
                StopCoroutine(holeCoroutines[1]);
                holeCoroutines[1] = holeCoroutines[0];
                holeCoroutines[0] = coroutine;
            }
        }
        public void ReleaseGas(Gas gas, float amount, float radius, Vector2 point, Vector2 velocity, float wind, float diffuseRate, float concentrationMultiplier)
        {
            if (!CanReleaseGas())
                return;
            float releasedAmount = Mathf.Min(gases[gas], amount);
            GameObject gasObj = new GameObject("gas");
            gasObj.transform.localScale = new Vector3(radius * 2, radius * 2);
            gasObj.transform.parent = transform;
            // move to front sorting layer
            SpriteRenderer sr = gasObj.AddComponent<SpriteRenderer>();
            sr.sprite = Mod.gasSprite;
            sr.color = gas.color;
            sr.sortingLayerName = "Foreground";
            sr.sortingOrder = 10;
            gasObj.transform.localPosition = new Vector3(point.x, point.y, 0);
            GasBehaviour spawnedGas = gasObj.AddComponent<GasBehaviour>();
            spawnedGas.gas = gas;
            spawnedGas.wind = wind;
            spawnedGas.diffuseRate = diffuseRate;
            spawnedGas.velocity = transform.TransformDirection(velocity);
            spawnedGas.amount = amount;
            spawnedGas.spriteRenderer = sr;
            spawnedGas.concentrationMultiplier = concentrationMultiplier;
            spawnedGas.parent = gameObject;
            if (!infiniteGas)
                RemoveGas(gas, releasedAmount);
            CircleCollider2D col = gasObj.AddComponent<CircleCollider2D>();
            col.radius = radius;
            spawnedGas.collider = col;
            gasObj.transform.parent = null;
        }
        public void ReleaseGas(Gas gas, float amount)
        {
            if (!CanReleaseGas())
                return;
            float releasedAmount = Mathf.Min(gases[gas], amount);
            GameObject gasObj = new GameObject("gas");
            gasObj.transform.localScale = new Vector3(spawnRadius * 2, spawnRadius * 2);
            gasObj.transform.parent = transform;
            // move to front sorting layer
            SpriteRenderer sr = gasObj.AddComponent<SpriteRenderer>();
            sr.sprite = Mod.gasSprite;
            sr.color = gas.color;
            sr.sortingLayerName = "Foreground";
            sr.sortingOrder = 10;
            gasObj.transform.localPosition = new Vector3(releasePoint.x, releasePoint.y, 0);
            GasBehaviour spawnedGas = gasObj.AddComponent<GasBehaviour>();
            spawnedGas.gas = gas;
            spawnedGas.wind = Random.Range(windMin, windMin);
            spawnedGas.diffuseRate = Random.Range(gas.diffuseRateMin, gas.diffuseRateMax);
            spawnedGas.velocity = transform.TransformDirection(releaseVelocity); // set this to be relative to rotation!
            spawnedGas.amount = releasedAmount;
            spawnedGas.spriteRenderer = sr;
            spawnedGas.parent = gameObject;
            if (!infiniteGas)
                RemoveGas(gas, releasedAmount);
            CircleCollider2D col = gasObj.AddComponent<CircleCollider2D>();
            col.radius = spawnRadius;
            spawnedGas.collider = col;
            gasObj.transform.parent = null;
        }
        public override void FillGas(string gasName)
        {
            base.FillGas(gasName);
            currentGas = gasName;
        }
        void ContextMenu()
        {
            GetComponent<PhysicalBehaviour>().ContextMenuOptions.Buttons.Add(new ContextMenuButton("windbut", "Edit direction and speed", "Change how fast the gas moves.", new UnityAction[1]
            {
                () =>
                {
                    DialogBox dialog = null;
                    dialog = DialogBoxManager.TextEntry("Change how fast the gas moves. Negative values will cause the gas to move left. Speed is randomized, so you have to input two values. Default values are 0.5 and 1.", "Min, Max", new DialogButton("Apply", true, new UnityAction[1]
                    {
                        () =>
                        {
                            if (dialog.EnteredText != "")
                            {;
                                var text = dialog.EnteredText.Split(","[0]);

                                if (text.Length == 2)
                                {
                                    ModAPI.Notify("Speed set to " + dialog.EnteredText);
                                    windMin = float.Parse(text[0]);
                                    windMax = float.Parse(text[1]);
                                }
                                else
                                    ModAPI.Notify("You did not input two values or did not put a comma. Speed unchanged.");
                            }
                            else
                                ModAPI.Notify("You didn't input anything. Speed unchanged.");
                        }
                    }),
                    new DialogButton("Cancel", true, new UnityAction[1]
                    {
                        () => dialog.Close()
                    }));
                }
            }));

            GetComponent<PhysicalBehaviour>().ContextMenuOptions.Buttons.Add(new ContextMenuButton("gasbut", "Change gas type", "Switch what type of gas is in the canister.", new UnityAction[1]
            {
                () =>
                {
                    butPos = 0;
                    CheckDialogArea(null);
                }
            }));

            this.GetComponent<PhysicalBehaviour>().ContextMenuOptions.Buttons.Add(new ContextMenuButton("mutebut", audio.enabled ? "Mute" : "Unmute", "Make the canister shut up or make it not shut up.", new UnityAction[1]
            {
                () =>
                {
                    if (audio.enabled)
                        audio.enabled = false;
                    else
                        audio.enabled = true;
                }
            }));
            this.GetComponent<PhysicalBehaviour>().ContextMenuOptions.Buttons.Add(new ContextMenuButton("refbut", "Refill canister", "Refill the canister.", new UnityAction[1]
            {
                () =>
                {
                    FillGas(currentGas);
                }
            }));
            this.GetComponent<PhysicalBehaviour>().ContextMenuOptions.Buttons.Add(new ContextMenuButton("capbut", "Edit capacity", "Change how much gas the canister can store.", new UnityAction[1]
            {
                () =>
                {
                    DialogBox dialog = null;
                    dialog = DialogBoxManager.TextEntry("Change the amount of gas that can be stored in the canister. Current is " + gasCapacity + ".", "", new DialogButton("Apply", true, new UnityAction[1]
                    {
                        () =>
                        {
                            if (dialog.EnteredText != "")
                            {;
                                gasCapacity = int.Parse(dialog.EnteredText);
                                ModAPI.Notify("Gas amount set to " + int.Parse(dialog.EnteredText).ToString());
                            }
                            else
                                ModAPI.Notify("You didn't input anything. Gas capacity unchanged.");
                            }
                    }),
                    new DialogButton(infiniteGas ? "Disable Infinite Gas" : "Enable Infinite Gas", true, new UnityAction[1]
                    {
                        () => 
                        {
                            if (!infiniteGas)
                            {
                                infiniteGas = true;
                                FillGas(currentGas);
                            }
                            else
                                infiniteGas = false;
                        }
                    }),
                new DialogButton("Cancel", true, new UnityAction[1]
                    {
                        () => dialog.Close()
                    }));
                }
            }));
            GetComponent<PhysicalBehaviour>().ContextMenuOptions.Buttons.Add(new ContextMenuButton("bulletbut", "Toggle bulletproof", "Toggle whether or not shooting the canister creates holes.", new UnityAction[1]
            {
                () =>
                {
                    if (bulletproof)
                        bulletproof = false;
                    else
                        bulletproof = true;
                }
            }));
        }
        void CheckDialogArea(DialogBox dialogBox)
        {
            DialogBox dialog = null;
            switch (butPos)
            {
                case 0:
                    dialog = DialogBoxManager.TextEntry("Switch the type of gas that is in the canister.", "",
                    new DialogButton("<<", true, new UnityAction[1]
                    {
                        (() =>
                        {
                            if (butPos == 0)
                            {
                                dialog.Close();
                                return;
                            }
                            butPos--;
                            CheckDialogArea(dialog);
                        })
                    }),
                    new DialogButton("Mustard Gas", true, new UnityAction[1]
                    {
                        () =>
                        {
                            FillGas("mustard gas");
                        }
                    }),
                    new DialogButton("Chlorine Gas", true, new UnityAction[1]
                    {
                        () =>
                        {
                            FillGas("chlorine gas");
                        }
                    }),
                    new DialogButton(">>", true, new UnityAction[1]
                    {
                        (() =>
                        {
                            if (butPos == 2)
                                return;
                            butPos++;
                            CheckDialogArea(dialog);
                        })
                    }));
                    dialog.ShowTextBox = false;
                    dialogBox = dialog;
                    break;
                case 1:
                    dialog = DialogBoxManager.TextEntry("Switch the type of gas that is in the canister.", "",
                    new DialogButton("<<", true, new UnityAction[1]
                    {
                        (() =>
                        {
                            if (butPos == 0)
                                return;
                            butPos--;
                            CheckDialogArea(dialog);
                        })
                    }),
                    new DialogButton("Hydrogen Cyanide", true, new UnityAction[1]
                    {
                        () =>
                        {
                            FillGas("hydrogen cyanide");
                        }
                    }),
                    new DialogButton("Knockout Gas", true, new UnityAction[1]
                    {
                        () =>
                        {
                            FillGas("knockout gas");
                        }
                    }),
                    new DialogButton(">>", true, new UnityAction[1]
                    {
                        () =>
                        {
                            if (butPos == 2)
                                return;
                            butPos++;
                            CheckDialogArea(dialog);
                        }
                    }));
                    dialog.ShowTextBox = false;
                    dialogBox = dialog;
                    break;
                case 2:
                    dialog = DialogBoxManager.TextEntry("Switch the type of gas that is in the canister.", "",
                    new DialogButton("<<", true, new UnityAction[1]
                    {
                        () =>
                        {
                            if (butPos == 0)
                                return;
                            butPos--;
                            CheckDialogArea(dialog);
                        }
                    }),
                    new DialogButton("Zombie Neurotoxin", true, new UnityAction[1]
                    {
                        () =>
                        {
                            FillGas("zombie neurotoxin");
                        }
                    }),
                    new DialogButton(">>", true, new UnityAction[1]
                    {
                        () =>
                        {
                            if (butPos == 2)
                            {
                                dialog.Close();
                                return;
                            }
                            butPos++;
                            CheckDialogArea(dialog);
                        }
                    }));
                    dialog.ShowTextBox = false;
                    dialogBox = dialog;
                    break;
            }
        }
    }
}