using System;
using UnityEngine;

namespace Mod
{
    // code for all the symptoms that can be applied
    public class Symptom : ScriptableObject
    {
        public void Init(string name, float chance, float mild, float moderate, float severe, float time)
        {
            this.name = name;
            this.chance = chance;
            this.mild = mild;
            this.moderate = moderate;
            this.severe = severe;
            this.time = time;
        }
        // whether or not the symptom is active
        public bool active = false;

        // just the name of the symptom
        public string name;

        // base severity multipliers
        // you can change these around to make certain effects stronger at certain levels
        // setting the value to 0 means the symptom will not occur at that level
        // symptoms can disappear if it is not set to take effect in higher exposure levels
        public float mild = 0.5f;
        public float moderate = 1;
        public float severe = 2;

        // if the symptom is one that always occurs or a one time change
        //public bool persistent = true;

        // amount of time the symptom will last after exposure ends
        // Any negative number will make it last forever, 0 makes it end right as exposure ends
        // this is affected by multipliers
        public float time = 180f;

        // chance of the symptom occuring
        // 1 = 100% chance
        // this is affected by multipliers
        public float chance = 1;
        // these affect the chance of the symptom occuring
        public float skinExposureMultiplier = 1;
        public float inhalationExposureMultiplier = 1;
        // multiplier for when victim is not being actively exposed to the gas
        public float nonexposedMultiplier = 0.5f;

        // makes the symptom fluctuate in severity
        // zero means it wont fluctuate at all
        public float fluctuationMagnitude = 0;
        public float fluctuationFrequency = 0;

        // amount of time after required exposure level that the symptom will take effect
        // 0 means instantaneous
        // affected by multipliers
        public float delay = 0;

        // effects
        // may be affected by multipliers
        public Func<float, float, float> burnDamageEffect; // one float for the value and one for multiplier

        public Func<float, float, float> breathTimeEffect;
        public Func<float, float, float> breathIntakeEffect;
        public Func<float, float, float> breathRandomizationEffect;

        public float spasmIntensity = 0;
        public float spasmChance = 0;

        public Func<float, float, float> strengthEffect;
        public Func<float, float, float> acidEffect;
        public Func<LimbBehaviour, float, float> healthEffect;

        public Func<float, float, float> consciousnessEffect;
        public Func<float, float, float> shockEffect;
        public Func<float, float, float> lungEffect;

        public Func<float, float, float> rottenEffect;

        public Action<GameObject, float>[] otherEffects;
    }
}