using UnityEngine;

namespace Mod
{
    public class GasBehaviour : MonoBehaviour
    {
        [SkipSerialisation]
        public Gas gas;

        public float amount;
        public float concentration; // amount divided by area. for a circle this would be (pi * radius)^2

        [SkipSerialisation]
        public CircleCollider2D collider;
        [SkipSerialisation]
        public SpriteRenderer spriteRenderer;
        public GameObject parent;

        [SkipSerialisation]
        public float wind = 0;
        float timePassed;
        float initialRadius;
        [SkipSerialisation]
        public float diffuseRate;
        [SkipSerialisation]
        public float concentrationMultiplier = 1f;

        public Vector2 velocity;
        private float rotation = Random.Range(1f, 5f);

        void Start()
        {
            initialRadius = collider.radius;
        }
        void FixedUpdate()
        {
            timePassed += Time.deltaTime;

            Vector2 nextVelocity = velocity + new Vector2(wind * Time.deltaTime, gas.buoyancy * Time.deltaTime);
            float magnitude = (nextVelocity * Time.deltaTime).magnitude;

            LayerMask mask = LayerMask.GetMask("Objects", "Bounds");
            RaycastHit2D hit = Physics2D.Raycast(transform.position, nextVelocity, magnitude, mask);
            if (!hit)
                hit = Physics2D.Raycast(transform.position, new Vector2(nextVelocity.x, -nextVelocity.y), magnitude, mask);
            if (hit)
            {
                if (!hit.collider.GetComponent<LimbBehaviour>() && !hit.collider.GetComponent<CanisterBehaviour>() && !hit.collider.GetComponent<ArmorBehaviour>())
                {
                    velocity = Vector3.Reflect(velocity, hit.normal); // this is not how physics works!
                    Vector2 damp = velocity * (Vector2.one - new Vector2(Mathf.Min(Mathf.Abs(hit.normal.x), 0.5f), Mathf.Min(Mathf.Abs(hit.normal.y), 0.5f)));
                    velocity = damp;
                }
            }

            velocity += new Vector2(wind * Time.deltaTime, gas.buoyancy * Time.deltaTime);
            transform.position += (Vector3)velocity * Time.deltaTime;

            float radius = Mathf.Sqrt(((diffuseRate * timePassed) + initialRadius) / Mathf.PI);
            float area = Mathf.PI * Mathf.Pow(radius, 2);
            transform.localScale = new Vector3(2 * radius, 2 * radius);
            transform.rotation *= Quaternion.Euler(0, 0, rotation * Time.deltaTime);
            concentration = (amount * concentrationMultiplier) / area;
            Color col = spriteRenderer.color;
            col.a = Mathf.Clamp(concentration - 0.05f, 0, 1);
            spriteRenderer.color = col;

            if (!parent || concentration - 0.05f <= 0)
                Destroy(gameObject);
        }
    }
    public struct Gas
    {
        public Gas(float buoyancy, float diffuseRateMin, float diffuseRateMax, Color color, Symptom[] symptoms)
        {
            this.buoyancy = buoyancy;
            this.diffuseRateMin = diffuseRateMin;
            this.diffuseRateMax = diffuseRateMax;
            this.color = color;
            this.symptoms = symptoms;
        }
        public float buoyancy; // default -0.25f
        public float diffuseRateMin;
        public float diffuseRateMax;
        public Color color;
        public Symptom[] symptoms; // remember to create new
    }
}