using System.Collections.Generic;
using UnityEngine;
using System.Linq;

namespace Mod
{
    public class GasContainer : MonoBehaviour
    {
        public float gasCapacity;
        public float GasAmount()
        {
            if (gases.Count == 0)
                return 0;
            return gases.Values.Sum();
        }
        public Dictionary<Gas, float> gases = new Dictionary<Gas, float>();
        public bool transfersGas;

        public void AddGas(Gas gas, float amount)
        {
            float addedAmount = Mathf.Min(gasCapacity - GasAmount(), amount);
            if (!gases.ContainsKey(gas))
                gases.Add(gas, 0);
            gases[gas] += addedAmount;
        }
        public virtual void FillGas(Gas gas)
        {
            gases.Clear();
            gases.Add(gas, gasCapacity);
        }
        public virtual void FillGas(string gasName)
        {
            gases.Clear();
            gases.Add(Mod.gas[gasName], gasCapacity);
        }
        public void RemoveGas(Gas gas, float amount)
        {
            // checks if it has the gas being removed
            if (!gases.ContainsKey(gas))
                return;
            gases[gas] -= amount; // this can go below zero
            if (gases[gas] <= 0)
                gases.Remove(gas);
        }
        // removes some amount of every gas
        public void DispenseAllGas(float amount)
        {
            Dictionary<Gas, float> oldGases = gases.ShallowClone();
            foreach (KeyValuePair<Gas, float> kvp in oldGases)
            {
                RemoveGas(kvp.Key, amount);
            }    
        }
        public void RemoveAllGas(Gas gas)
        {
            gases.Remove(gas);
        }
        public virtual void AbsorbGases(float amount)
        {
            Collider2D[] colliders = Physics2D.OverlapCircleAll(transform.position, transform.localScale.magnitude);
            if (colliders.Length == 0)
                return;
            foreach (Collider2D collider in colliders)
            {
                if (collider.gameObject.TryGetComponent(out GasBehaviour gas))
                {
                    float absorbedAmount = Mathf.Min(amount * gas.concentration, gas.amount);
                    AddGas(gas.gas, absorbedAmount);
                    gas.amount -= absorbedAmount;
                }
            }
        }
    }
}