using System;
using System.Collections.Generic;
using System.Collections;
using UnityEngine;

namespace Mod
{
    public class SymptomBehaviour : GasContainer
    {
        public bool immune;
        public List<Gas> exposed = new List<Gas>();
        public List<Symptom> activeSymptoms = new List<Symptom>();
        [SkipSerialisation]
        public LimbBehaviour limb;
        [SkipSerialisation]
        public PersonBehaviour person;
        public List<Collider2D> colliders = new List<Collider2D>();
        public virtual void Start()
        {
            StartCoroutine(ContinousAbsorbGases());
        }
        public virtual void FixedUpdate()
        {
            if (immune)
                return;
            foreach (KeyValuePair<Gas, float> gas in gases)
            {
                foreach (Symptom symptom in gas.Key.symptoms)
                {
                    if (!activeSymptoms.Contains(symptom))
                        StartCoroutine(DoSymptom(symptom, gas.Key));
                }
            }
        }
        private void OnWillRenderObject()
        {
            ModAPI.Draw.Circle(transform.position, 0.3f);
        }
        IEnumerator ContinousAbsorbGases()
        {
            while (true && !immune)
            {
                yield return new WaitForSeconds(0.5f);
                AbsorbGases(.035f);
                DispenseAllGas(.005f);
            }
        }
        public virtual IEnumerator DoSymptom(Symptom symptom, Gas gas)
        {
            bool active = false;
            float delayPassed = 0;
            float timePassed = 0;
            float multiplier = 1;

            void UpdateMultiplier()
            {
                if (!gases.ContainsKey(gas))
                    multiplier = symptom.mild;
                else if (gases[gas] >= 0.85f)
                    multiplier = symptom.severe;
                else if (gases[gas] >= 0.333f)
                    multiplier = symptom.moderate;
                else
                    multiplier = symptom.mild;
                if (!exposed.Contains(gas))
                    multiplier *= symptom.nonexposedMultiplier;
            }
            activeSymptoms.Add(symptom);

            float chance = symptom.chance * Time.fixedDeltaTime * symptom.skinExposureMultiplier;
            while (!active) // skin absorption
            {
                if (!gases.ContainsKey(gas))
                    break;
                if (UnityEngine.Random.value < chance * multiplier) // chance for a symptom to occur on any body part is multiplied by 14
                    active = true;
                UpdateMultiplier();
                yield return new WaitForFixedUpdate();
            }
            //float otherTime = 0; // debug
            while (active && !immune)
            {
                UpdateMultiplier();
                // need to add: fluctuation stuff
                while (multiplier == 0)
                {
                    UpdateMultiplier();
                    if (!gases.ContainsKey(gas))
                        active = false;
                    yield return new WaitForFixedUpdate();
                }
                while (symptom.delay > 0 && multiplier != 0 && delayPassed < (symptom.delay / multiplier))
                {
                    UpdateMultiplier();
                    delayPassed += Time.fixedDeltaTime;
                    // Debug.Log("delayed " + symptom.name + " " + delayPassed + "/" + symptom.delay / multiplier);
                    yield return new WaitForFixedUpdate();
                }
                //otherTime += Time.deltaTime;
                //if (otherTime >= 1) // debug
                //{
                //    Debug.Log(symptom.name);
                //    otherTime = 0;
                //}
                if (symptom.time >= 0 && !gases.ContainsKey(gas))
                {
                    timePassed += Time.fixedDeltaTime;
                    if (timePassed >= symptom.time)
                        active = false;
                }

                if (symptom.burnDamageEffect != null)
                    limb.PhysicalBehaviour.BurnProgress = symptom.burnDamageEffect(limb.PhysicalBehaviour.BurnProgress, multiplier);
                if (symptom.acidEffect != null)
                    limb.SkinMaterialHandler.AcidProgress = symptom.acidEffect(limb.SkinMaterialHandler.AcidProgress, multiplier);
                if (symptom.consciousnessEffect != null)
                    person.Consciousness = symptom.consciousnessEffect(person.Consciousness, multiplier);
                if (symptom.healthEffect != null)
                    limb.Health = symptom.healthEffect(limb, multiplier);
                if (UnityEngine.Random.value <= symptom.spasmChance * Time.fixedDeltaTime)
                    limb.Wince(symptom.spasmIntensity);
                if (symptom.shockEffect != null)
                    person.ShockLevel = symptom.shockEffect(person.ShockLevel, multiplier);

                if (symptom.otherEffects != null)
                {
                    foreach (Action<GameObject, float> action in symptom.otherEffects)
                    {
                        action(limb.gameObject, multiplier);
                    }
                }
                yield return new WaitForFixedUpdate();
            }
            yield return new WaitWhile(() => gases.ContainsKey(gas)); // made while tired, make sure it actually fits this correctly
            activeSymptoms.Remove(symptom);
        }
        public override void AbsorbGases(float amount)
        {
            ContactFilter2D filter = new ContactFilter2D();
            filter.layerMask = 1 << 0;
            int count = Physics2D.OverlapCircle(transform.position, 0.3f, filter, colliders);
            exposed.Clear();
            for (int i = 0; i < count - 1; i++)
            {
                Collider2D collider = colliders[i];
                if (collider.gameObject.TryGetComponent(out GasBehaviour gas))
                {
                    float absorbedAmount = Mathf.Min(amount * Mathf.Min(gas.concentration, 1), gas.amount);
                    AddGas(gas.gas, absorbedAmount);
                    gas.amount -= absorbedAmount;

                    if (!exposed.Contains(gas.gas))
                        exposed.Add(gas.gas);
                }
            }
        }
    }
}